"use strict";
/*!
 * Copyright (c) 2020 Ron Buckton (rbuckton@chronicles.org)
 *
 * This file is licensed to you under the terms of the MIT License, found in the LICENSE file
 * in the root of this repository or package.
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.GrammarkdownEmitter = void 0;
const emitter_1 = require("./emitter");
const tokens_1 = require("../tokens");
/** {@docCategory Emit} */
class GrammarkdownEmitter extends emitter_1.Emitter {
    constructor() {
        super(...arguments);
        this.extension = ".grammar";
    }
    emitNode(node) {
        if (node && tokens_1.isTokenKind(node.kind)) {
            return this.emitToken(node);
        }
        switch (node === null || node === void 0 ? void 0 : node.kind) {
            case tokens_1.SyntaxKind.StringLiteral: return this.emitStringLiteral(node);
            case tokens_1.SyntaxKind.NumberLiteral: return this.emitNumberLiteral(node);
        }
        return super.emitNode(node);
    }
    emitSourceFile(node) {
        var _a;
        let lastElementWasMeta = false;
        let lastCollapsedProduction;
        let hasWrittenElement = false;
        for (const element of node.elements) {
            const thisElementIsMeta = tokens_1.isMetaElementKind(element.kind);
            const thisCollapsedProduction = element.kind === tokens_1.SyntaxKind.Production && ((_a = element.body) === null || _a === void 0 ? void 0 : _a.kind) === tokens_1.SyntaxKind.RightHandSide ? element : undefined;
            if (hasWrittenElement) {
                if (!(thisElementIsMeta && lastElementWasMeta) &&
                    !(thisCollapsedProduction && lastCollapsedProduction && thisCollapsedProduction.name.text === lastCollapsedProduction.name.text)) {
                    this.writer.commitLine();
                    this.writer.writeln();
                }
            }
            this.emitNode(element);
            lastElementWasMeta = thisElementIsMeta;
            lastCollapsedProduction = thisCollapsedProduction;
            hasWrittenElement = true;
        }
        this.writer.writeln();
    }
    emitStringLiteral(node) {
        var _a;
        this.writer.write(JSON.stringify((_a = node.text) !== null && _a !== void 0 ? _a : ""));
    }
    emitNumberLiteral(node) {
        this.emitTextContent(node);
    }
    emitProduction(node) {
        var _a, _b, _c;
        this.emitIdentifier(node.name);
        this.emitNode(node.parameterList);
        this.writer.write(" ");
        this.emitTokenKind((_b = (_a = node.colonToken) === null || _a === void 0 ? void 0 : _a.kind) !== null && _b !== void 0 ? _b : tokens_1.SyntaxKind.ColonToken);
        switch ((_c = node.body) === null || _c === void 0 ? void 0 : _c.kind) {
            case tokens_1.SyntaxKind.OneOfList:
            case tokens_1.SyntaxKind.RightHandSide:
                this.writer.write(" ");
                break;
        }
        this.emitNode(node.body);
        this.writer.writeln();
    }
    emitParameterList(node) {
        this.writer.write("[");
        if (node.elements) {
            for (let i = 0; i < node.elements.length; ++i) {
                if (i > 0) {
                    this.writer.write(`, `);
                }
                this.emitNode(node.elements[i]);
            }
        }
        this.writer.write(`]`);
    }
    emitParameter(node) {
        this.emitIdentifier(node.name);
    }
    emitOneOfList(node) {
        var _a, _b, _c, _d, _e, _f, _g;
        this.emitTokenKind(tokens_1.SyntaxKind.OneKeyword);
        this.writer.write(" ");
        this.emitTokenKind(tokens_1.SyntaxKind.OfKeyword);
        if ((_a = node.terminals) === null || _a === void 0 ? void 0 : _a.length) {
            let maxLength = 1;
            for (const child of node.terminals) {
                const len = (_c = (_b = child.text) === null || _b === void 0 ? void 0 : _b.length) !== null && _c !== void 0 ? _c : 0;
                if (len > maxLength)
                    maxLength = len;
            }
            const columnCount = Math.max(1, Math.floor(56 / (maxLength + 2)));
            this.writer.writeln();
            this.writer.indent();
            const columnSizes = Array(columnCount).fill(0);
            for (let i = 0; i < node.terminals.length; i++) {
                columnSizes[i % columnCount] = Math.max(columnSizes[i % columnCount], (_e = (_d = node.terminals[i].text) === null || _d === void 0 ? void 0 : _d.length) !== null && _e !== void 0 ? _e : 0);
            }
            for (let i = 0; i < node.terminals.length; i++) {
                if (i > 0) {
                    if ((i % columnCount) === 0) {
                        this.writer.writeln();
                    }
                    else {
                        const lastColumnSize = columnSizes[(i % columnCount) - 1];
                        const lastLen = (_g = (_f = node.terminals[i - 1].text) === null || _f === void 0 ? void 0 : _f.length) !== null && _g !== void 0 ? _g : 1;
                        this.writer.write(" ".repeat(lastColumnSize - lastLen + 1));
                    }
                }
                this.emitNode(node.terminals[i]);
            }
            this.writer.dedent();
        }
    }
    emitRightHandSideList(node) {
        this.writer.indent();
        if (node.elements) {
            for (const rhs of node.elements) {
                this.writer.writeln();
                this.emitNode(rhs);
            }
        }
        this.writer.dedent();
    }
    emitRightHandSide(node) {
        this.emitChildren(node);
    }
    emitSymbolSpan(node) {
        this.emitNode(node.symbol);
        if (node.next) {
            this.writer.write(" ");
            this.emitNode(node.next);
        }
    }
    emitPlaceholder(node) {
        this.emitTokenKind(tokens_1.SyntaxKind.AtToken);
    }
    emitTerminal(node) {
        this.emitNode(node.literal);
        this.emitNode(node.questionToken);
    }
    emitNonterminal(node) {
        this.emitChildren(node);
    }
    emitArgumentList(node) {
        this.writer.write("[");
        if (node.elements) {
            for (let i = 0; i < node.elements.length; ++i) {
                if (i > 0) {
                    this.writer.write(`, `);
                }
                this.emitNode(node.elements[i]);
            }
        }
        this.writer.write(`]`);
    }
    emitArgument(node) {
        this.emitToken(node.operatorToken);
        this.emitNode(node.name);
    }
    emitUnicodeCharacterRange(node) {
        this.emitTextContent(node.left);
        this.writer.write(` through `);
        this.emitTextContent(node.right);
    }
    emitUnicodeCharacterLiteral(node) {
        this.emitTextContent(node);
    }
    emitTerminalLiteral(node) {
        var _a;
        if (node.text === "`") {
            this.writer.write("```");
        }
        else {
            this.writer.write("`");
            this.writer.write((_a = node.text) !== null && _a !== void 0 ? _a : "");
            this.writer.write("`");
        }
    }
    emitProse(node) {
        this.writer.write("> ");
        node.fragments && this.emitNodes(node.fragments);
    }
    emitEmptyAssertion(node) {
        this.writer.write("[empty]");
    }
    emitSymbolSet(node) {
        this.writer.write(`{`);
        if (node.elements) {
            for (let i = 0; i < node.elements.length; ++i) {
                if (i > 0) {
                    this.writer.write(`,`);
                }
                this.writer.write(` `);
                this.emitNode(node.elements[i]);
            }
        }
        this.writer.write(` }`);
    }
    emitLookaheadAssertion(node) {
        var _a, _b;
        this.writer.write(`[`);
        this.emitToken(node.lookaheadKeyword);
        this.writer.write(" ");
        this.emitTokenKind((_b = (_a = node.operatorToken) === null || _a === void 0 ? void 0 : _a.kind) !== null && _b !== void 0 ? _b : tokens_1.SyntaxKind.EqualsToken);
        this.writer.write(" ");
        this.emitNode(node.lookahead);
        this.writer.write(`]`);
    }
    emitLexicalGoalAssertion(node) {
        this.writer.write(`[lexical goal `);
        this.emitNode(node.symbol);
        this.writer.write(`]`);
    }
    emitNoSymbolHereAssertion(node) {
        this.writer.write(`[no `);
        if (node.symbols) {
            for (let i = 0; i < node.symbols.length; ++i) {
                if (i > 0) {
                    this.writer.write(` or `);
                }
                this.emitNode(node.symbols[i]);
            }
        }
        this.writer.write(` here]`);
    }
    emitConstraints(node) {
        this.writer.write("[");
        if (node.elements) {
            for (let i = 0; i < node.elements.length; ++i) {
                if (i > 0) {
                    this.writer.write(`, `);
                }
                this.emitNode(node.elements[i]);
            }
        }
        this.writer.write("] ");
    }
    emitProseAssertion(node) {
        this.writer.write(`[>`);
        if (node.fragments) {
            for (const fragment of node.fragments) {
                this.emitNode(fragment);
            }
        }
        this.writer.write(`]`);
    }
    emitButNotSymbol(node) {
        this.emitNode(node.left);
        this.writer.write(` but not `);
        this.emitNode(node.right);
    }
    emitOneOfSymbol(node) {
        this.writer.write(`one of `);
        if (node.symbols) {
            for (let i = 0; i < node.symbols.length; ++i) {
                if (i > 0) {
                    this.writer.write(` or `);
                }
                this.emitNode(node.symbols[i]);
            }
        }
    }
    emitTextContent(node) {
        if (node === null || node === void 0 ? void 0 : node.text) {
            this.writer.write(node.text);
        }
    }
    emitDefine(node) {
        this.emitNode(node.atToken);
        this.emitNode(node.defineKeyword);
        this.writer.write(" ");
        this.emitNode(node.key);
        this.writer.write(" ");
        this.emitNode(node.valueToken);
        this.writer.writeln();
    }
    emitLine(node) {
        this.emitNode(node.atToken);
        this.emitNode(node.lineKeyword);
        this.writer.write(" ");
        this.emitNode(node.number);
        if (node.path) {
            this.writer.write(" ");
            this.emitNode(node.path);
        }
        this.writer.writeln();
    }
    emitImport(node) {
        this.emitNode(node.atToken);
        this.emitNode(node.importKeyword);
        this.writer.write(" ");
        this.emitNode(node.path);
        this.writer.writeln();
    }
}
exports.GrammarkdownEmitter = GrammarkdownEmitter;
//# sourceMappingURL=grammarkdown.js.map